"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeDataSource = exports.AppDataSource = void 0;
const bcryptjs_1 = require("bcryptjs");
const typeorm_1 = require("typeorm");
const model_1 = require("./model");
exports.AppDataSource = new typeorm_1.DataSource({
    type: 'sqlite',
    database: 'db/device.db',
    synchronize: true,
    entities: [model_1.ScopeModel, model_1.RoleModel, model_1.UserModel, model_1.KeyModel, model_1.ActiveKeyModel, model_1.TokenModel],
});
async function createScopes(scopes) {
    const scopeCollection = {
        all: [],
        user: [],
        developer: [],
        auth_service: [],
        device_service: [],
        experiment_service: [],
        federation_service: [],
        update_service: [],
    };
    const scopeRepository = exports.AppDataSource.getRepository(model_1.ScopeModel);
    for (const scope of scopes) {
        const scopeModel = scopeRepository.create();
        scopeModel.name = scope.name;
        await scopeRepository.save(scopeModel);
        scopeCollection.all.push(scopeModel);
        if (scope.roles === 'all') {
            scopeCollection.developer.push(scopeModel);
            scopeCollection.user.push(scopeModel);
            scopeCollection.auth_service.push(scopeModel);
            scopeCollection.device_service.push(scopeModel);
            scopeCollection.experiment_service.push(scopeModel);
            scopeCollection.federation_service.push(scopeModel);
            scopeCollection.update_service.push(scopeModel);
        }
        else {
            for (const role of scope.roles.filter((v, i, s) => s.indexOf(v) === i)) {
                switch (role) {
                    case 'developer':
                        scopeCollection.developer.push(scopeModel);
                        break;
                    case 'user':
                        scopeCollection.user.push(scopeModel);
                        break;
                    case 'auth_service':
                        scopeCollection.auth_service.push(scopeModel);
                        break;
                    case 'device_service':
                        scopeCollection.device_service.push(scopeModel);
                        break;
                    case 'experiment_service':
                        scopeCollection.experiment_service.push(scopeModel);
                        break;
                    case 'federation_service':
                        scopeCollection.federation_service.push(scopeModel);
                        break;
                    case 'update_service':
                        scopeCollection.update_service.push(scopeModel);
                        break;
                }
            }
        }
    }
    return scopeCollection;
}
async function createRole(name, scopes) {
    const roleRepository = exports.AppDataSource.getRepository(model_1.RoleModel);
    const existingRole = await roleRepository.findOneBy({ name: name });
    if (existingRole === null) {
        const role = roleRepository.create();
        role.name = name;
        role.scopes = scopes;
        await roleRepository.save(role);
    }
    else {
        existingRole.scopes = scopes;
        await roleRepository.save(existingRole);
    }
}
async function createDefaultScopesAndRoles() {
    // create default scopes
    const scopeCollection = await createScopes([
        { name: 'authorized_proxy', roles: 'all' },
        { name: 'device', roles: 'all' },
        { name: 'device:create', roles: 'all' },
        { name: 'device:connect', roles: 'all' },
        { name: 'device:edit', roles: 'all' },
        { name: 'device:list', roles: 'all' },
        { name: 'experiment', roles: 'all' },
        { name: 'experiment:create', roles: 'all' },
        { name: 'experiment:edit', roles: 'all' },
        { name: 'experiment:list', roles: 'all' },
        { name: 'identity', roles: 'all' },
        { name: 'identity:edit', roles: 'all' },
        { name: 'identity:list', roles: 'all' },
        { name: 'institution', roles: 'all' },
        { name: 'institution:create', roles: 'all' },
        { name: 'institution:edit', roles: 'all' },
        { name: 'institution:list', roles: 'all' },
        { name: 'peerconnection', roles: 'all' },
        { name: 'peerconnection:create', roles: 'all' },
        { name: 'peerconnection:list', roles: 'all' },
        { name: 'update', roles: 'all' },
        { name: 'update:create', roles: 'all' },
        { name: 'update:edit', roles: 'all' },
        { name: 'update:list', roles: 'all' },
        { name: 'users', roles: 'all' },
        { name: 'users:create', roles: 'all' },
        { name: 'users:edit', roles: 'all' },
        { name: 'users:list', roles: 'all' },
        { name: 'device_token', roles: 'all' },
        { name: 'device_token:create', roles: 'all' },
        { name: 'logout', roles: 'all' },
    ]);
    // create default roles
    await createRole('superadmin', scopeCollection.all);
    await createRole('developer', scopeCollection.developer);
    await createRole('user', scopeCollection.user);
    await createRole('auth_service', scopeCollection.auth_service);
    await createRole('device_service', scopeCollection.device_service);
    await createRole('experiment_service', scopeCollection.experiment_service);
    await createRole('federation_service', scopeCollection.federation_service);
    await createRole('update_service', scopeCollection.update_service);
}
async function createDefaultSuperadminUser() {
    const userRepository = exports.AppDataSource.getRepository(model_1.UserModel);
    const roleRepository = exports.AppDataSource.getRepository(model_1.RoleModel);
    const roleSuperadmin = await roleRepository.findOneOrFail({
        where: {
            name: 'superadmin',
        },
        relations: {
            users: true,
        },
    });
    if (roleSuperadmin.users.length === 0) {
        const user = userRepository.create();
        user.username = 'superadmin';
        user.password = await (0, bcryptjs_1.hash)('superadmin', 10);
        user.roles = [roleSuperadmin];
        user.tokens = [];
        await userRepository.save(user);
    }
}
async function createDefaultServiceUser(service) {
    const userRepository = exports.AppDataSource.getRepository(model_1.UserModel);
    const roleRepository = exports.AppDataSource.getRepository(model_1.RoleModel);
    const roleAuthService = await roleRepository.findOneOrFail({
        where: {
            name: service,
        },
        relations: {
            users: true,
            scopes: true,
        },
    });
    if (roleAuthService.users.length === 0) {
        const user = userRepository.create();
        user.username = service.replace('_', '');
        user.roles = [roleAuthService];
        user.tokens = [];
        await userRepository.save(user);
    }
}
async function initializeDataSource() {
    await createDefaultScopesAndRoles();
    await createDefaultSuperadminUser();
    await createDefaultServiceUser('auth_service');
    await createDefaultServiceUser('device_service');
    await createDefaultServiceUser('experiment_service');
    await createDefaultServiceUser('federation_service');
    await createDefaultServiceUser('update_service');
}
exports.initializeDataSource = initializeDataSource;
//# sourceMappingURL=data_source.js.map