"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTokenByTokenString = exports.getTokenStringFromAuthorization = exports.signDeviceToken = exports.signUserToken = exports.sign = exports.getAllowlistedUser = void 0;
const model_1 = require("../model");
const jose_1 = require("jose");
const config_1 = require("../config");
const data_source_1 = require("../data_source");
const errors_1 = require("../types/errors");
const __1 = require("..");
/**
 * Try to find user associated to allowlisted IP.
 * @param ip IP from the client that is potentially allowlisted.
 * @throws {MissingEntityError} User associated with allowlisted IP needs to exist in the database.
 * @returns The user associated with the allowlisted IP.
 */
async function getAllowlistedUser(ip) {
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    console.warn(`IP ${ip} is allowlisted, trying to find associated user ${__1.allowlist[ip]}`);
    const user = await userRepository.findOne({
        where: {
            username: __1.allowlist[ip],
        },
        relations: {
            roles: {
                scopes: true,
            },
        },
    });
    if (!user)
        throw new errors_1.MissingEntityError(`User ${__1.allowlist[ip]} for allowlisted IP ${ip} is not in the database`, 500);
    return user;
}
exports.getAllowlistedUser = getAllowlistedUser;
/**
 * This function signs a JWT.
 * @param payload Payload of the JWT.
 * @param key Key used for signing.
 * @param expirationTime Expiration time of the JWT.
 * @returns The signed JWT.
 */
async function sign(payload, key, expirationTime) {
    return await new jose_1.SignJWT(payload)
        .setProtectedHeader({ alg: key.alg, kid: key.uuid })
        .setIssuer(config_1.config.SECURITY_ISSUER)
        .setIssuedAt()
        .setAudience(config_1.config.SECURITY_AUDIENCE)
        .setExpirationTime(expirationTime)
        .sign(await (0, jose_1.importJWK)(JSON.parse(key.private_key), key.alg));
}
exports.sign = sign;
/**
 * This function signs a token for the user for the use of the microservice architecture.
 * @param user User for which the token should be signed.
 * @param activeKey Active key used for signing the token.
 * @returns The signed token.
 */
async function signUserToken(user, activeKey) {
    const BASE_URL = !config_1.config.BASE_URL.endsWith('/')
        ? config_1.config.BASE_URL
        : config_1.config.BASE_URL + '/';
    return await sign({
        url: BASE_URL + `/users/${user.username}`,
        username: user.username,
        scopes: user.roles
            .map((r) => r.scopes.map((s) => s.name))
            .flat(1)
            .filter((v, i, s) => s.indexOf(v) === i),
    }, activeKey.key, '2h');
}
exports.signUserToken = signUserToken;
/**
 * This function signs a token for the device for the use of the microservice architecture.
 * @param deviceUrl The url of the device for which the token should be signed.
 * @param user User for which the token should be signed.
 * @param activeKey Active key used for signing the token.
 * @returns The signed token.
 */
async function signDeviceToken(deviceUrl, user, activeKey) {
    const BASE_URL = !config_1.config.BASE_URL.endsWith('/')
        ? config_1.config.BASE_URL
        : config_1.config.BASE_URL + '/';
    return await sign({
        url: BASE_URL + `/users/${user.username}`,
        username: user.username,
        device: deviceUrl,
        scopes: user.roles
            .map((r) => r.scopes.map((s) => s.name))
            .flat(1)
            .filter((v, i, s) => s.indexOf(v) === i),
    }, activeKey.key, '2h');
}
exports.signDeviceToken = signDeviceToken;
/**
 * This function parses the string representation of a token from the Authorization parameter
 * @param authorization Authorization parameter from request.
 * @returns String representation of the token.
 */
function getTokenStringFromAuthorization(authorization) {
    if (!authorization) {
        // TODO: was - throw new MissingParameterError(`Authorization parameter is missing`, 401)
        throw new Error(`Authorization parameter is missing`);
    }
    const splitAuthorization = authorization.split(' ');
    if (splitAuthorization.length !== 2) {
        // TODO: was - throw new MalformedParameterError(`Authorization parameter is malformed`, 401)
        throw new Error(`Authorization parameter is malformed`);
    }
    return splitAuthorization[1];
}
exports.getTokenStringFromAuthorization = getTokenStringFromAuthorization;
/**
 * This function searches the database for a token matching the provided string.
 * @param tokenString String representation of the token.
 * @returns Token matching the provided string.
 */
async function getTokenByTokenString(tokenString) {
    const tokenRepository = data_source_1.AppDataSource.getRepository(model_1.TokenModel);
    const token = await tokenRepository.findOne({
        where: {
            token: tokenString,
        },
        relations: {
            user: {
                roles: {
                    scopes: true,
                },
            },
        },
    });
    if (!token) {
        throw new errors_1.MissingEntityError(`No matching token found for provided tokenString`, 404);
    }
    return token;
}
exports.getTokenByTokenString = getTokenByTokenString;
//# sourceMappingURL=auth.js.map