"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addRole = exports.isRequiredUser = exports.jwk = exports.generateNewKey = exports.resolveAllowlistEntry = void 0;
const jose_1 = require("jose");
const data_source_1 = require("../data_source");
const model_1 = require("../model");
const dns_1 = __importDefault(require("dns"));
const errors_1 = require("../types/errors");
/**
 * This function tries to resolve an entry of the allowlist.
 * @param entry Entry of the allowlist ("url":username") to be resolved.
 * @returns Resolved entry in the form "ip:username"
 */
async function resolveAllowlistEntry(entry) {
    console.log(`resolveAllowlistEntry called for "${entry}"`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    let url = '';
    let username = '';
    // Split provided entry into its url and username components
    if (entry.includes(':')) {
        const split = entry.split(':');
        if (split.length === 2) {
            ;
            [url, username] = split;
        }
        else {
            throw new errors_1.MalformedAllowlistEntryError(`Allowlist entry "${entry}" does not conform to format "url:username"`);
        }
    }
    if (!url)
        throw new errors_1.MalformedAllowlistEntryError(`Could not extract url from entry`);
    if (!username)
        throw new errors_1.MalformedAllowlistEntryError(`Could not extract username from allowlist entry`);
    // Resolve the ip of the provided url
    const ip = await new Promise((res) => {
        dns_1.default.lookup(url, (err, address) => {
            if (err || address.length === 0 || !address)
                return res('');
            return res(address);
        });
    });
    if (!ip)
        throw new errors_1.DNSResolveError(`Could not resolve ip for "${url}"`);
    // Search the user with the provided username
    const user = await userRepository.findOne({
        where: {
            username: username,
        },
        relations: {
            roles: {
                scopes: true,
            },
        },
    });
    if (!user)
        throw new errors_1.MissingEntityError(`Could not find user ${username}`);
    console.log(`resolveAllowlistEntry succeeded`);
    return [ip, user.username];
}
exports.resolveAllowlistEntry = resolveAllowlistEntry;
/**
 * This function generates a new key.
 * @param usage The usage of the key (default: "sig")
 * @returns The newly generated key.
 */
async function generateNewKey(usage = 'sig') {
    const keyRepository = data_source_1.AppDataSource.getRepository(model_1.KeyModel);
    const keyPair = await (0, jose_1.generateKeyPair)('RS256');
    const key = keyRepository.create();
    key.private_key = JSON.stringify(await (0, jose_1.exportJWK)(keyPair.privateKey));
    key.public_key = JSON.stringify(await (0, jose_1.exportJWK)(keyPair.publicKey));
    key.use = usage;
    key.alg = 'RS256';
    await keyRepository.save(key);
    return key;
}
exports.generateNewKey = generateNewKey;
/**
 * This function parses a JWK from the provided key.
 * @param key The key to be parsed.
 * @returns The parsed JWK.
 */
function jwk(key) {
    const jwk = JSON.parse(key.public_key);
    jwk.use = key.use;
    jwk.alg = key.alg;
    jwk.kid = key.uuid;
    return jwk;
}
exports.jwk = jwk;
/**
 * This function checks if a user has values for all possible properties.
 * @param user The user to be checked.
 * @returns True if user has values for all possible properties, else false.
 */
function isRequiredUser(user) {
    return (user !== undefined &&
        user.username !== undefined &&
        user.password !== undefined &&
        user.roles !== undefined);
}
exports.isRequiredUser = isRequiredUser;
/**
 * This function adds a role to an user.
 * @param user The UserModel the role should be added to.
 * @param role The RoleModel to be added.
 */
function addRole(user, role) {
    if (!user.roles.find((r) => r.name === role.name)) {
        user.roles.push(role);
    }
}
exports.addRole = addRole;
//# sourceMappingURL=utils.js.map