"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createUser = exports.writeUser = void 0;
const bcryptjs_1 = require("bcryptjs");
const data_source_1 = require("../data_source");
const model_1 = require("../model");
const errors_1 = require("../types/errors");
const utils_1 = require("./utils");
/**
 * This function writes the data of an User to an UserModel
 * @param userModel The UserModel the data should be written to.
 * @param user The User providing the data to be written.
 * @throws {InvalidValueError} Throws an error when the provided user has unknown roles.
 */
async function writeUser(userModel, user) {
    if (user.username)
        userModel.username = user.username;
    if (user.password)
        userModel.password = await (0, bcryptjs_1.hash)(user.password, 10);
    if (user.roles) {
        userModel.roles = [];
        const roleRepository = data_source_1.AppDataSource.getRepository(model_1.RoleModel);
        for (const role of user.roles) {
            const roleModel = await roleRepository.findOneBy({ name: role.name });
            if (!roleModel)
                throw new errors_1.InvalidValueError(`Requested role is not known: ${role.name}`, 400);
            (0, utils_1.addRole)(userModel, roleModel);
        }
    }
}
exports.writeUser = writeUser;
/**
 * This function creates a new user.
 * @param userdata The data to be used for creating the new user.
 * @throws {InvalidValueError} Can throw errors from {@link writeUser}
 * @returns The newly created user.
 */
async function createUser(userdata) {
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const user = userRepository.create();
    await writeUser(user, userdata);
    await userRepository.save(user);
    return user;
}
exports.createUser = createUser;
//# sourceMappingURL=write.js.map