"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAuth = void 0;
const data_source_1 = require("../data_source");
const model_1 = require("../model");
const __1 = require("..");
const auth_1 = require("../methods/auth");
const errors_1 = require("../types/errors");
/**
 * This function implements the functionality for handling GET requests on /auth endpoint.
 * @param parameters The parameters of the request.
 * @throws {MissingEntityError} Thrown if token is not found in database. | Can throw errors from {@link getAllowlistedUser}. (currently all caught)
 * @throws {InconsistentDatabaseError} Thrown if multiple active keys are found.
 */
const getAuth = async (parameters) => {
    console.log(`getAuth called`);
    const HOUR = 60 * 60 * 1000;
    const activeKeyRepository = data_source_1.AppDataSource.getRepository(model_1.ActiveKeyModel);
    const tokenRepository = data_source_1.AppDataSource.getRepository(model_1.TokenModel);
    // Catch missing authorization header (OPTIONS requests)
    if (!parameters.Authorization) {
        return {
            status: 200,
            headers: {}
        };
    }
    // Get active key
    const activeKeys = await activeKeyRepository.find({ relations: { key: true } });
    if (activeKeys.length != 1) {
        throw new errors_1.InconsistentDatabaseError('Too many active keys', 500);
    }
    const activeKey = activeKeys[0];
    // Non Allowlisted Auth
    try {
        // Resolve user from Authorization parameter
        const tokenString = (0, auth_1.getTokenStringFromAuthorization)(parameters.Authorization);
        const token = await (0, auth_1.getTokenByTokenString)(tokenString);
        const user = token.user;
        if (!user)
            throw new errors_1.MissingEntityError(`Token has no associated user`);
        // Check if token is expired
        if (token.expiresOn && new Date(token.expiresOn).getTime() < Date.now())
            throw new errors_1.ExpiredError(`Token is expired`);
        let jwt;
        // Check if token has a device
        if (token.device) {
            // Sign device token
            console.log(`signing jwt for device ${token.device}`);
            jwt = await (0, auth_1.signDeviceToken)(token.device, user, activeKey);
        }
        else {
            // Sign user token
            console.log(`signing jwt for user ${user.username}`);
            jwt = await (0, auth_1.signUserToken)(user, activeKey);
        }
        // Update token expiration time
        if (token.expiresOn)
            token.expiresOn = new Date(Date.now() + HOUR).toISOString();
        tokenRepository.save(token);
        console.log(`getAuth succeeded`);
        return {
            status: 200,
            headers: {
                Authorization: 'Bearer ' + jwt,
            },
        };
    }
    catch (error) {
        // Allowlisted Auth
        if (parameters['X-Real-IP'] && __1.allowlist[parameters['X-Real-IP']]) {
            try {
                const user = await (0, auth_1.getAllowlistedUser)(parameters['X-Real-IP']);
                console.log(`signing jwt for user ${user.username}`);
                const jwt = await (0, auth_1.signUserToken)(user, activeKey);
                console.log(`getAuth succeeded`);
                return {
                    status: 200,
                    headers: {
                        Authorization: 'Bearer ' + jwt,
                    },
                };
            }
            catch (error) {
                console.error(`Authentication of allowlisted IP failed: ${error}`);
                return {
                    status: 500,
                };
            }
        }
        else {
            console.error(`getAuth failed: ${error}`);
            return {
                status: 500,
            };
        }
    }
};
exports.getAuth = getAuth;
//# sourceMappingURL=auth.js.map