"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteUsersByUsernameRolesByRoleName = exports.putUsersByUsernameRolesByRoleName = exports.patchUsersByUsername = exports.deleteUsersByUsername = exports.getUsersByUsername = exports.postUsers = exports.getUsers = void 0;
const model_1 = require("../model");
const data_source_1 = require("../data_source");
const errors_1 = require("../types/errors");
const service_common_1 = require("@crosslab/service-common");
const format_1 = require("../methods/format");
const utils_1 = require("../methods/utils");
const write_1 = require("../methods/write");
/**
 * This function implements the functionality for handling GET requests on /users endpoint.
 * @param _user The user submitting the request.
 */
const getUsers = async (_user) => {
    console.log(`getUsers called`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const users = await userRepository.find({
        relations: {
            roles: {
                scopes: true,
            },
        },
    });
    console.log(`getUsers succeeded`);
    return {
        status: 200,
        body: users.map(format_1.formatUser),
    };
};
exports.getUsers = getUsers;
/**
 * This function implements the functionality for handling POST requests on /users endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MalformedBodyError} Thrown if the body is missing required properties.
 * @throws {InvalidValueError} Can throw errors from {@link createUser}.
 */
const postUsers = async (body, _user) => {
    console.log(`postUsers called`);
    if (!(0, utils_1.isRequiredUser)(body))
        throw new service_common_1.MalformedBodyError(`Body is missing required properties`, 400);
    const user = await (0, write_1.createUser)(body);
    console.log(`postUsers succeeded`);
    return {
        status: 201,
        body: (0, format_1.formatUser)(user),
    };
};
exports.postUsers = postUsers;
/**
 * This function implements the functionality for handling GET requests on /users/{username} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 */
const getUsersByUsername = async (parameters, _user) => {
    console.log(`getUsersByUsername called`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const user = await userRepository.findOne({
        where: {
            username: parameters.username,
        },
        relations: {
            roles: {
                scopes: true,
            },
        },
    });
    if (!user)
        throw new errors_1.MissingEntityError(`Could not find user ${parameters.username}`, 404);
    console.log(`getUsersByUsername succeeded`);
    return {
        status: 200,
        body: (0, format_1.formatUser)(user),
    };
};
exports.getUsersByUsername = getUsersByUsername;
/**
 * This function implements the functionality for handling DELETE requests on /users/{username} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 */
const deleteUsersByUsername = async (parameters, _user) => {
    console.log(`deleteUsersByUsername called`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const user = await userRepository.findOneBy({ username: parameters.username });
    if (!user)
        throw new errors_1.MissingEntityError(`Could not find user ${parameters.username}`, 404);
    await userRepository.softDelete(user);
    console.log(`deleteUsersByUsername succeeded`);
    return {
        status: 204,
    };
};
exports.deleteUsersByUsername = deleteUsersByUsername;
/**
 * This function implements the functionality for handling PATCH requests on /users/{username} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 * @throws {InvalidValueError} Can throw errors from {@link writeUser}.
 */
const patchUsersByUsername = async (parameters, body, _user) => {
    console.log(`patchUsersByUsername called`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const user = await userRepository.findOneBy({ username: parameters.username });
    if (!user)
        throw new errors_1.MissingEntityError(`Could not find user ${parameters.username}`, 404);
    await (0, write_1.writeUser)(user, body ?? {});
    await userRepository.save(user);
    console.log(`patchUsersByUsername succeeded`);
    return {
        status: 200,
        body: (0, format_1.formatUser)(user),
    };
};
exports.patchUsersByUsername = patchUsersByUsername;
/**
 * This function implements the functionality for handling PUT requests on /users/{username}/roles/{role_name} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user or role is not found in the database.
 */
const putUsersByUsernameRolesByRoleName = async (parameters, _user) => {
    console.log(`putUsersByUsernameRolesByRoleName called`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const roleRepository = data_source_1.AppDataSource.getRepository(model_1.RoleModel);
    const user = await userRepository.findOneBy({ username: parameters.username });
    const role = await roleRepository.findOneBy({ name: parameters.role_name });
    if (!user)
        throw new errors_1.MissingEntityError(`Could not find user ${parameters.username}`, 404);
    if (!role)
        throw new errors_1.MissingEntityError(`Could not find role ${parameters.role_name}`, 404);
    (0, utils_1.addRole)(user, role);
    await userRepository.save(user);
    console.log(`putUsersByUsernameRolesByRoleName succeeded`);
    return {
        status: 200,
        body: (0, format_1.formatUser)(user),
    };
};
exports.putUsersByUsernameRolesByRoleName = putUsersByUsernameRolesByRoleName;
/**
 * This function implements the functionality for handling DELETE requests on /users/{username}/roles/{role_name} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request
 * @throws {MissingEntityError} Thrown if user or role is not found in the database.
 */
const deleteUsersByUsernameRolesByRoleName = async (parameters, _user) => {
    console.log(`deleteUsersByUsernameRolesByRoleName called`);
    const userRepository = data_source_1.AppDataSource.getRepository(model_1.UserModel);
    const roleRepository = data_source_1.AppDataSource.getRepository(model_1.RoleModel);
    const user = await userRepository.findOneBy({ username: parameters.username });
    const role = await roleRepository.findOneBy({ name: parameters.role_name });
    if (!user)
        throw new errors_1.MissingEntityError(`Could not find user ${parameters.username}`, 404);
    if (!role)
        throw new errors_1.MissingEntityError(`Could not find role ${parameters.role_name}`, 404);
    let index = user.roles.findIndex((r) => r.name === parameters.role_name);
    while (index !== -1) {
        user.roles.splice(index, 1);
        index = user.roles.findIndex((r) => r.name === parameters.role_name);
    }
    await userRepository.save(user);
    console.log(`deleteUsersByUsernameRolesByRoleName succeeded`);
    return {
        status: 204,
    };
};
exports.deleteUsersByUsernameRolesByRoleName = deleteUsersByUsernameRolesByRoleName;
//# sourceMappingURL=users.js.map